/*****************************************************************************/
/*                                                                           */
/*                                                                           */
/* Chaotic pendulum                                                          */
/* ================                                                          */
/*                                                                           */
/* A twodimensional frictionless chaotic pendulum consisting of two          */
/* connected rotating homogenous bar masses with different lengths           */
/* and initial angles.                                                       */
/*                                                                           */
/* The author does not know if the equations are equavilent also for         */
/* for massless bars with point masses at the end of the bars. If mass       */
/* circles are displayed they are shown at the outer ends of the bars.       */
/*                                                                           */
/* Set graphics resolution, console mode or window mode, background colour   */  
/* etc and the pendulum parameters in the file "chaopend2.cfg".              */
/*                                                                           */
/* Note. This version of the program can run several chaotic pendulums       */
/* at the same time. This is also set in the file "chaopend2.cfg".           */
/*                                                                           */
/* The RGB color palette for the program is now located in the file          */
/* "chaopend2-colors.cfg".                                                   */
/*                                                                           */
/* Terminate the program by typing the key "q" (quit).                       */
/*                                                                           */
/*                                                                           */
/* Author etc                                                                */
/* ==========                                                                */
/*                                                                           */
/* Program version: chaopend2.c 1.60                                         */
/*                                                                           */
/* The program is free to copy and modify.                                   */
/*                                                                           */
/* Author: Stefan Spaennare, Lund, Sweden                                    */
/*                                                                           */
/* E-mail: stefan@spaennare.se                                               */
/*                                                                           */
/* Home page:    http://www.spaennare.se/index.html                          */
/* Program page: http://www.spaennare.se/ssprog2.html                        */
/*                                                                           */
/* C version for Windows: 2006-07-04;   Using Allegro game library and       */
/*                                      Microsoft Visual C++ 6.0.            */
/* Latest update:         2012-12-29;                                        */
/*                                                                           */
/*                                                                           */
/* References                                                                */
/* ==========                                                                */
/*                                                                           */
/* 1. The chaotic pendulum model and the equations for it were found at      */
/*    the University museum at Kulturen in Lund, Sweden.                     */
/*                                                                           */
/* 2. The program uses a classical 4th order Runge-Kutta method for          */
/*    integration of the differential equations.                             */
/*                                                                           */
/* 3. The program uses a random number generator ran2() from Numerical       */
/*    Recipes in C Second Edition.                                           */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "allegro.h"


double pi=3.14159265358979323;

double timetic=(double)(CLOCKS_PER_SEC);

char cfg_name[21]="chaopend2.cfg";

char color_name[21]="chaopend2-colors.cfg";

char image_name[21]="chaopend2.bmp";


#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */


void comp_acc(acc1,acc2)
double *acc1;
double *acc2;
{
   double v1,v2,m1,m2,l1,l2,g,a1,a2,a3;
   double d1,d1t,d2,d2t,sind1d2,deninv;
     
      d1=acc1[1];  d2=acc1[2];
      d1t=acc1[3]; d2t=acc1[4];
      l1=acc1[5];  l2=acc1[6];
      m1=acc1[7];  m2=acc1[8];
      g=acc1[9];

      sind1d2=sin(d1-d2);

      v1=-(m1+m2)*g*l1*sin(d1)-m2*l1*l2*d2t*d2t*sind1d2;
      v2=-m2*g*l2*sin(d2)+m2*l1*l2*d1t*d1t*sind1d2;

      a1=(m1+m2)*l1*l1;
      a2=m2*l1*l2*cos(d1-d2);
      a3=m2*l2*l2;

      deninv=1.0/(a1*a3-a2*a2);

      acc2[1]=deninv*(a3*v1-a2*v2);
      acc2[2]=deninv*(a1*v2-a2*v1);

} /* comp_acc */


void runge_kutta(acc1,acc2)
double *acc1;
double *acc2;
{

   double aa1,aa2,bb1,bb2,cc1,cc2,dd1,dd2;
   double beta1,beta2,delta1,delta2,k11,k12,k21,k22;   
   double h,h2,inv3,d1,d2,d1t,d2t;

      d1=acc1[1];  d2=acc1[2];
      d1t=acc1[3]; d2t=acc1[4];

      h=acc1[10];
      h2=acc1[11];
      inv3=acc1[12];

      comp_acc(acc1,acc2);

      aa1=h2*acc2[1];
      aa2=h2*acc2[2];

      beta1=h2*(d1t+0.5*aa1);
      beta2=h2*(d2t+0.5*aa2);

      acc1[1]=d1+beta1; acc1[2]=d2+beta2;
      acc1[3]=d1t+aa1;  acc1[4]=d2t+aa2;

      comp_acc(acc1,acc2);

      bb1=h2*acc2[1];
      bb2=h2*acc2[2];

      acc1[1]=d1+beta1; acc1[2]=d2+beta2;
      acc1[3]=d1t+bb1;  acc1[4]=d2t+bb2;
      
      comp_acc(acc1,acc2);

      cc1=h2*acc2[1];
      cc2=h2*acc2[2];

      delta1=h*(d1t+cc1);
      delta2=h*(d2t+cc2);

      acc1[1]=d1+delta1;   acc1[2]=d2+delta2;
      acc1[3]=d1t+2.0*cc1; acc1[4]=d2t+2.0*cc2;

      comp_acc(acc1,acc2);

      dd1=h2*acc2[1];
      dd2=h2*acc2[2];

      k11=inv3*(aa1+bb1+cc1);
      k12=inv3*(aa2+bb2+cc2);

      k21=inv3*(aa1+2.0*bb1+2.0*cc1+dd1);
      k22=inv3*(aa2+2.0*bb2+2.0*cc2+dd2);

      d1=d1+h*(d1t+k11);
      d2=d2+h*(d2t+k12);

      d1t=d1t+k21;
      d2t=d2t+k22;

      acc1[1]=d1;  acc1[2]=d2;
      acc1[3]=d1t; acc1[4]=d2t;

} /*  runge_kutta */


void disp_pend(acc1,pd,xxi,vdisp,cold,cr,cg,cb)
double *acc1;
double *pd;
int *xxi;
int *vdisp;
int *cold;
int *cr;
int *cg;
int *cb;
{
      int i,flag,standflag,borderflag;
      int dispstep,dispflag11,dispflag12,dispflag21,dispflag22;
      int bgcol,standcol,bordercol,col11,col12,col21,col22,c0,col;
      int x1i2,y1i2,x2i2,y2i2,x0i,y0i,x1i,y1i,x2i,y2i;
      int x1s,y1s,x2s,y2s,xsize,ysize,rcmi1,rcmi2;

      double d1,d2,x1,y1,x2,y2,l1,l2,w,xrf,yrf,lf;

      i=vdisp[1];
      dispstep=vdisp[3];

      if (i == dispstep) {

         xsize=xxi[1]; ysize=xxi[2];
         x0i=xxi[3]; y0i=xxi[4];
         x1i=xxi[5]; y1i=xxi[6];
         x2i=xxi[7]; y2i=xxi[8];
         x1i2=xxi[9]; y1i2=xxi[10];
         x2i2=xxi[11]; y2i2=xxi[12];
         x1s=xxi[13]; y1s=xxi[14];
         x2s=xxi[15]; y2s=xxi[16];
         rcmi1=xxi[17]; rcmi2=xxi[18];

         i=vdisp[1];
         flag=vdisp[2];
         dispstep=vdisp[3];
         standflag=vdisp[4];
         borderflag=vdisp[5];
         dispflag11=vdisp[6];
         dispflag12=vdisp[7];
         dispflag21=vdisp[8];
         dispflag22=vdisp[9];

         bgcol=cold[1];
         bordercol=cold[2];
         standcol=cold[3];
         col11=cold[4];
         col12=cold[5];
         col21=cold[6];
         col22=cold[7];

         l1=pd[1];
         l2=pd[2];
         xrf=pd[3];
         yrf=pd[4];
         lf=pd[5];
         w=pd[6];


         d1=acc1[1];  d2=acc1[2];

         x1=l1*cos(d1+w);
         y1=l1*sin(d1+w);

         x2=l2*cos(d2+w);
         y2=l2*sin(d2+w); 

         c0=bgcol;
         col=makecol(cr[c0],cg[c0],cb[c0]);

         if (dispflag12 == 1) {
            circle(screen,x1i,y1i,rcmi1,col);
         } /* if */

         c0=bgcol;
         col=makecol(cr[c0],cg[c0],cb[c0]);

         if (dispflag22 == 1) {
            circle(screen,x2i,y2i,rcmi2,col);
         } /* if */

         c0=bgcol;
         col=makecol(cr[c0],cg[c0],cb[c0]);

         if (dispflag11 == 1) {
            line(screen,x0i,y0i,x1i,y1i,col);
         } /* if */

         c0=bgcol;
         col=makecol(cr[c0],cg[c0],cb[c0]);

         if (dispflag21 == 1) {
            line(screen,x1i,y1i,x2i,y2i,col);
         } /* if */

         x1i=(int)(xrf*lf*x1+0.5)+x0i;
         y1i=-(int)(yrf*lf*y1+0.5)+y0i;

         x2i=(int)(xrf*lf*x2+0.5)+x1i;
         y2i=-(int)(yrf*lf*y2+0.5)+y1i;

         if (standflag == 1) {
            c0=standcol;
            col=makecol(cr[c0],cg[c0],cb[c0]);
            line(screen,x1s,ysize-1,x1s,y1s,col);
            line(screen,x2s,ysize-1,x2s,y2s,col);
            line(screen,x1s,y1s,x2s,y2s,col);
         } /* if */

         if (flag == 0) {
            x1i2=x1i;
            y1i2=y1i;
            x2i2=x2i;
            y2i2=y2i;
         } /* if */

         c0=col11;
         col=makecol(cr[c0],cg[c0],cb[c0]);

         if (dispflag11 == 1) {
            line(screen,x0i,y0i,x1i,y1i,col);
         } /* if */

         if (dispflag11 == 2) {
            putpixel(screen,x1i,y1i,col);
         } /* if */

         if (dispflag11 == 3) {
            line(screen,x1i2,y1i2,x1i,y1i,col);
         } /* if */

         c0=col21;
         col=makecol(cr[c0],cg[c0],cb[c0]);

         if (dispflag21 == 1) {
            line(screen,x1i,y1i,x2i,y2i,col);
         } /* if */

         if (dispflag21 == 2) {
            putpixel(screen,x2i,y2i,col);
         } /* if */

         if (dispflag21 == 3) {
            line(screen,x2i2,y2i2,x2i,y2i,col);
         } /* if */

         c0=col12;
         col=makecol(cr[c0],cg[c0],cb[c0]);

         if (dispflag12 == 1) {
            circle(screen,x1i,y1i,rcmi1,col);
         } /* if */

         c0=col22;
         col=makecol(cr[c0],cg[c0],cb[c0]);

         if (dispflag22 == 1) {
            circle(screen,x2i,y2i,rcmi2,col);
         } /* if */

         if (borderflag == 1) {
            c0=bordercol;
            col=makecol(cr[c0],cg[c0],cb[c0]);
            line(screen,0,0,xsize-1,0,col);
            line(screen,xsize-1,0,xsize-1,ysize-1,col);
            line(screen,xsize-1,ysize-1,0,ysize-1,col);
            line(screen,0,ysize-1,0,0,col);
         } /* if */

         x1i2=x1i;
         y1i2=y1i;
         x2i2=x2i;
         y2i2=y2i;

         i=0;
         flag=1;

         vdisp[1]=i;
         vdisp[2]=flag;

         xxi[5]=x1i;
         xxi[6]=y1i;
         xxi[7]=x2i;
         xxi[8]=y2i;

         xxi[9]=x1i2;
         xxi[10]=y1i2;
         xxi[11]=x2i2;
         xxi[12]=y2i2;

      } /* if */

} /* disp_pend */


int main(argc,argv)
int argc;
char *argv[];
{

   FILE *cfgfile;
   FILE *colorfile;

   BITMAP *bmp;
   PALETTE pal;

   long idum;

   int i,j,seed,delay0,npend,ti;
   int xsize,ysize,col,c0,wintype,imsave;
   int x0i1,y0i1,x1i1,y1i1,x2i1,y2i1;
   int x1i2,y1i2,x2i2,y2i2,x1s1,y1s1,x2s1,y2s1;
   int bgcol0,bordercol0,standcol0,borderflag0,rcmi1,rcmi2;
   int col11,col12,col21,col22;
   int dispflag11,dispflag12,dispflag21,dispflag22;
   int dispstep,standflag;
   int nacc,npd,nvdisp,nxxi,ncold,ncolor;

   double xrf,yrf,xs0,ys0,sum,totsum,ttt1,ttt2,g;
   double d1,d1t,d2,d2t,dt,rcm1,rcm2;
   double standw,standh,temp,h,h2,inv3,w;
   double deps1,deps2,xposf,yposf,lfac,lf;
   double m1,m2,l1,l2,d10,d20;

   double **acc1;
   double *mpacc1;
   double **acc2;
   double *mpacc2;

   double **pd;
   double *mppd;

   int **vdisp;
   int *mpvdisp;

   int **xxi;
   int *mpxxi;

   int **cold;
   int *mpcold;

   int *disppend;

   int *cr;
   int *cg;
   int *cb;

   char cc;

   time_t tt;

   clock_t time1,timediff1;


   /* Open input file. */

   if ((cfgfile = fopen(cfg_name,"r")) == NULL) {
      printf("File %s not found.\n",cfg_name);
      exit(0);
   } /* if */


   /* Read first data from input file. */

   fscanf(cfgfile,"%d %d",&xsize,&ysize);
   fscanf(cfgfile,"%d",&wintype);
   fscanf(cfgfile,"%d",&imsave);

   fscanf(cfgfile,"%d",&delay0);
   fscanf(cfgfile,"%lf",&dt);
   fscanf(cfgfile,"%lf",&g);

   fscanf(cfgfile,"%d",&bgcol0);
   fscanf(cfgfile,"%d",&bordercol0);
   fscanf(cfgfile,"%d",&standcol0);
   fscanf(cfgfile,"%d",&borderflag0);

   fscanf(cfgfile,"%d",&npend);

   if ((wintype < 0) || (wintype > 1)) {
      printf("wintype must be 0 or 1.\n");
      exit(0);
   } /* if */ 

   if ((imsave < 0) || (imsave > 1)) {
      printf("imsave must be 0 or 1.\n");
      exit(0);
   } /* if */ 

   if ((borderflag0 < 0) || (borderflag0 > 1)) {
      printf("borderflag must be 0 or 1.\n");
      exit(0);
   } /* if */ 

   if (npend < 1) {
      npend=1;
   } /* if */ 

   /* Init main arrays. */

   nacc=15;
   acc1=(double **)calloc(npend+1,sizeof(double*));
   mpacc1=(double *)calloc((npend+1)*(nacc+1),sizeof(double));
   acc2=(double **)calloc(npend+1,sizeof(double*));
   mpacc2=(double *)calloc((npend+1)*(nacc+1),sizeof(double));

   for (i=0; i<=npend; i++) {
      acc1[i]=mpacc1;
      mpacc1=mpacc1+nacc+1;
      acc2[i]=mpacc2;
      mpacc2=mpacc2+nacc+1;
   } /* for i */

   npd=11;
   pd=(double **)calloc(npend+1,sizeof(double*));
   mppd=(double *)calloc((npend+1)*(npd+1),sizeof(double));

   for (i=0; i<=npend; i++) {
      pd[i]=mppd;
      mppd=mppd+npd+1;
   } /* for i */

   nvdisp=21;
   vdisp=(int **)calloc(npend+1,sizeof(int*));
   mpvdisp=(int *)calloc((npend+1)*(nvdisp+1),sizeof(int));

   for (i=0; i<=npend; i++) {
      vdisp[i]=mpvdisp;
      mpvdisp=mpvdisp+nvdisp+1;
   } /* for i */

   nxxi=21;
   xxi=(int **)calloc(npend+1,sizeof(int*));
   mpxxi=(int *)calloc((npend+1)*(nxxi+1),sizeof(int));

   for (i=0; i<=npend; i++) {
      xxi[i]=mpxxi;
      mpxxi=mpxxi+nxxi+1;
   } /* for i */

   ncold=11;
   cold=(int **)calloc(npend+1,sizeof(int*));
   mpcold=(int *)calloc((npend+1)*(ncold+1),sizeof(int));

   for (i=0; i<=npend; i++) {
      cold[i]=mpcold;
      mpcold=mpcold+ncold+1;
   } /* for i */

   disppend=(int *)calloc(npend+1,sizeof(int));


   /* Open color file. */

   if ((colorfile = fopen(color_name,"r")) == NULL) {
      printf("File %s not found.\n",color_name);
      exit(0);
   } /* if */

   /* Set up color arrays and palette. */

   fscanf(colorfile,"%d",&ncolor);

   cr=(int *)calloc(ncolor+1,sizeof(int));
   cg=(int *)calloc(ncolor+1,sizeof(int));
   cb=(int *)calloc(ncolor+1,sizeof(int));

   for (i=0; i<ncolor; i++) {
      fscanf(colorfile,"%d %d %d",&cr[i],&cg[i],&cb[i]);
   } /* for i */


   /* Init Allegro fungctions and graphics. */

   allegro_init();

   install_keyboard();

   set_color_depth(32);


   if (wintype == 0) {
      if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            printf("Try to set resolution to 640 480.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */

   if (wintype == 1) {
      if (set_gfx_mode(GFX_AUTODETECT_WINDOWED,xsize,ysize,0,0) < 0) {
         if (set_gfx_mode(GFX_AUTODETECT,xsize,ysize,0,0) < 0) {
            allegro_exit();
            printf("Graphics error or wrong resolution.\n");
            printf("Try to set resolution to 640 480.\n");
            exit(0);
         } /* if */
      } /* if */
   } /* if */


   /* Clear background with apprpriate color. */

   c0=bgcol0;
   col=makecol(cr[c0],cg[c0],cb[c0]);

   rectfill(screen,0,0,xsize-1,ysize-1,col);


   /* Init random seed etc. */

   tt=time(0);
   ttt1=(double)(tt)+0.5;
   ttt2=fmod(ttt1,2147483647.0);
   seed=(long)(ttt2);
   idum=-seed-1;


   xrf=(double)(xsize)/640.0;
   yrf=(double)(ysize)/480.0;
   xs0=320.0;
   ys0=240.0;


   /* Read data from input file and setup pendulum runge kutta and display arrays. */

   for (i=1; i<=npend; i++) {

      fscanf(cfgfile,"%d",&col11);
      fscanf(cfgfile,"%d",&col12);
      fscanf(cfgfile,"%d",&col21);
      fscanf(cfgfile,"%d",&col22);

      fscanf(cfgfile,"%d",&ti); disppend[i]=ti;
      fscanf(cfgfile,"%d",&standflag);
      fscanf(cfgfile,"%d",&dispflag11);
      fscanf(cfgfile,"%d",&dispflag12);
      fscanf(cfgfile,"%d",&dispflag21);
      fscanf(cfgfile,"%d",&dispflag22);
      fscanf(cfgfile,"%d",&dispstep);

      fscanf(cfgfile,"%lf",&xposf);
      fscanf(cfgfile,"%lf",&yposf);
      fscanf(cfgfile,"%lf",&standw);
      fscanf(cfgfile,"%lf",&standh);

      fscanf(cfgfile,"%lf",&d10);
      fscanf(cfgfile,"%lf",&d20);
      fscanf(cfgfile,"%lf",&deps1);
      fscanf(cfgfile,"%lf",&deps2);

      fscanf(cfgfile,"%lf",&lfac);
      fscanf(cfgfile,"%lf",&l1);
      fscanf(cfgfile,"%lf",&l2);
      fscanf(cfgfile,"%lf",&rcm1);
      fscanf(cfgfile,"%lf",&rcm2);

      fscanf(cfgfile,"%lf",&m1);
      fscanf(cfgfile,"%lf",&m2);


   if ((standflag < 0) || (standflag > 1)) {
      printf("standflag must be 0 or 1.\n");
      exit(0);
   } /* if */ 

   if ((dispflag11 < 0) || (dispflag11 > 3)) {
      printf("dispflag11 must be 0, 1, 2 or 3.\n");
      exit(0);
   } /* if */ 

   if ((dispflag12 < 0) || (dispflag12 > 1)) {
      printf("dispflag12 must be 0 or 1.\n");
      exit(0);
   } /* if */ 

   if ((dispflag21 < 0) || (dispflag21 > 3)) {
      printf("dispflag21 must be 0, 1, 2 or 3.\n");
      exit(0);
   } /* if */ 

   if ((dispflag22 < 0) || (dispflag22 > 1)) {
      printf("dispflag22 must be 0 or 1.\n");
      exit(0);
   } /* if */ 


   x0i1=(int)(2.0*xrf*xs0*xposf+0.5);
   y0i1=(int)(2.0*yrf*ys0*(1.0-yposf)+0.5);
   x1i1=x0i1;
   y1i1=y0i1;
   x2i1=x0i1;
   y2i1=y0i1;

   x1s1=(int)(standw*xrf*xs0+0.5);
   y1s1=(int)(2.0*yrf*ys0*(1.0-yposf)-standh*yrf*ys0+0.5);
   x2s1=-x1s1+x0i1;
   x1s1=x1s1+x0i1;
   y2s1=y1s1;

   lf=lfac*ys0/(l1+l2);

   rcmi1=(int)(lf*yrf*rcm1+0.5);
   rcmi2=(int)(lf*yrf*rcm2+0.5);


   if (d10 < 0.0) {
      temp=fabs(d10);
      d10=(180.0-temp)*ran2(&idum)+temp;
   } /* if */

   if (d20 < 0.0) {
      temp=fabs(d20);
      d20=(180.0-temp)*ran2(&idum)+temp;
   } /* if */

   d1=(d10+deps1)*pi/180.0;
   d2=(d20+deps2)*pi/180.0;
   d1t=0.0;
   d2t=0.0;


   h=dt;
   h2=0.5*dt;

   inv3=1.0/3.0;

   w=-90.0*pi/180.0;


   acc1[i][1]=d1;  acc1[i][2]=d2;
   acc1[i][3]=d1t; acc1[i][4]=d2t;

   acc1[i][5]=l1; acc1[i][6]=l2;
   acc1[i][7]=m1; acc1[i][8]=m2;
   acc1[i][9]=g;

   acc1[i][10]=h;
   acc1[i][11]=h2;
   acc1[i][12]=inv3;

   vdisp[i][1]=1;
   vdisp[i][2]=0;
   vdisp[i][3]=dispstep;
   vdisp[i][4]=standflag;
   vdisp[i][5]=borderflag0;
   vdisp[i][6]=dispflag11;
   vdisp[i][7]=dispflag12;
   vdisp[i][8]=dispflag21;
   vdisp[i][9]=dispflag22;

   xxi[i][1]=xsize;  xxi[i][2]=ysize;
   xxi[i][3]=x0i1;    xxi[i][4]=y0i1;
   xxi[i][5]=x1i1;    xxi[i][6]=y1i1;
   xxi[i][7]=x2i1;    xxi[i][8]=y2i1;
   xxi[i][9]=x1i2;   xxi[i][10]=y1i2;
   xxi[i][11]=x2i2;  xxi[i][12]=y2i2;
   xxi[i][13]=x1s1;   xxi[i][14]=y1s1;
   xxi[i][15]=x2s1;   xxi[i][16]=y2s1;
   xxi[i][17]=rcmi1; xxi[i][18]=rcmi2;

   cold[i][1]=bgcol0;
   cold[i][2]=bordercol0;
   cold[i][3]=standcol0;
   cold[i][4]=col11;
   cold[i][5]=col12;
   cold[i][6]=col21;
   cold[i][7]=col22;

   pd[i][1]=l1;
   pd[i][2]=l2;
   pd[i][3]=xrf;
   pd[i][4]=yrf;
   pd[i][5]=lf;
   pd[i][6]=w;

   } /* for i */


   totsum=0.0;


   /* Start of main integration loop. */
    
   do {

      for (i=1; i<=npend; i++) {

         if (disppend[i] == 1) {

             runge_kutta(acc1[i],acc2[i]); 

             disp_pend(acc1[i],pd[i],xxi[i],vdisp[i],cold[i],cr,cg,cb);

             vdisp[i][1]++;

         } /* if */

      } /* for i */

      sum=0.0;

      for (j=1; j<=delay0; j++) {
         sum=sum+1.0/(double)(j);
      } /* for j */

      totsum=totsum+sum;

      if (keypressed()) {
         cc=(readkey() & 0xff);
      } /* if */
   

   } while (cc != 'q');

   /* End of main loop. */


   /* Save bitmap screen dump if desired. */

   if (imsave == 1) {
      get_palette(pal);
      bmp=create_sub_bitmap(screen,0,0,xsize,ysize);
      save_bitmap(image_name,bmp,pal);
      destroy_bitmap(bmp);
   } /* if */

  
   /* Exit allegro and close files. */

   allegro_exit();

   fclose(cfgfile);

   fclose(colorfile);

   printf("dummy = %12.6f\n",totsum);

   return(0);

} /* End  */

END_OF_MAIN();
